<?php
namespace WilokeListgoFunctionality\Framework\Payment\PayPal;

use PayPal\Api\Amount;
use PayPal\Api\Details;
use PayPal\Api\Item;
use PayPal\Api\ItemList;
use PayPal\Api\Payer;
use PayPal\Api\Payment;
use PayPal\Api\RedirectUrls;
use PayPal\Api\Transaction;
use PayPal\Exception\PayPalConnectionException;

use WilokeListgoFunctionality\Framework\Payment\PaymentMethodInterface;
use WilokeListgoFunctionality\Framework\Payment\Coupon;
use WilokeListgoFunctionality\Framework\Payment\PaymentConfiguration;
use WilokeListgoFunctionality\Framework\Store\Session;

use WilokeListgoFunctionality\Model\PaymentModel;


class PayPalNonRecurringPaymentMethod implements PaymentMethodInterface{
	use PayPalGenerateUrls;
	use PayPalSetup;

	public $gateway = 'paypal';
	protected $storeTokenPlanSession;
	protected $aConfiguration = array();
	protected $oReceipt;
	protected $oApiContext = array();
	protected $paymentDescription = 'AgreePayment'; // This description is very important if you are using recurring payment
	protected $thankyouUrl = null;
	protected $cancelUrl = null;
	public $token = null;
	protected $aPayPayConfiguration;
	protected $instPayPalConfiguration;

	public function getBillingType() {
		return wilokeRepository('app:billingTypes', true)->sub('nonrecurring');
	}

	public function proceedPayment($oReceipt){
		$this->setup($oReceipt);
		$aResult = $this->getApprovalUrl();
		return $aResult;
	}

	private function parseTokenFromApprovalUrl($approvalUrl){
		$aParseData = explode('token=', $approvalUrl);
		$this->token = trim($aParseData[1]);
	}

	private function storeTokenAndPlanId(){
		if ( !$this->oReceipt->isMultiplePlan ){
			Session::setSession($this->storeTokenPlanSession, array(
				$this->token => $this->oReceipt->planID
			));
		}else{
			Session::setSession($this->storeTokenPlanSession, array(
				$this->token => $this->oReceipt->aPlans
			));
		}
	}

	private function getApprovalUrl(){
		// Create new payer and method
		$payer = new Payer();
		$payer->setPaymentMethod('paypal');
		$aItems = array();
		
		$instPlan = new Item();
		$instPlan->setName($this->oReceipt->planName)
		         ->setCurrency($this->aConfiguration['currency_code'])
		         ->setQuantity(1)
		         ->setPrice($this->oReceipt->subTotal);
		$aItems[] = $instPlan;

		if ( !empty($this->oReceipt->aCouponInfo['discountPrice']) ){
			// Maybe discount
			$discountPrice = '-' . $this->oReceipt->aCouponInfo['discountPrice'];
			settype($discountPrice, 'string');

			$insDiscount = new Item();
			$insDiscount->setName($this->oReceipt->aCouponInfo['planName'])
			            ->setCurrency($this->aConfiguration['currency_code'])
			            ->setQuantity(1)
			            ->setPrice($discountPrice);
			$aItems[] = $insDiscount;
		}

		if ( isset($discountPrice) ){
			$total = $this->oReceipt->subTotal - $this->oReceipt->aCouponInfo['discountPrice'];
		}else{
			$total = $this->oReceipt->subTotal;
		}

		$instItemList = new ItemList();
		$instItemList->setItems($aItems);

		$instDetails = new Details();
		$instDetails->setSubtotal($total)->setShipping(0)->setTax(0);

		$instAmount = new Amount();
		$instAmount->setCurrency($this->aConfiguration['currency_code'])
		       ->setDetails($instDetails);

		if ( isset($discountPrice) ){
			$instAmount->setTotal($total);
		}else{
			$instAmount->setTotal($total);
		}

		// Set transaction object
		$transaction = new Transaction();
		$transaction->setItemList($instItemList)
					->setAmount($instAmount)
					->setInvoiceNumber(uniqid())
		            ->setDescription($this->paymentDescription);

		// Set redirect urls
		$redirectUrls = new RedirectUrls();
		$redirectUrls->setReturnUrl($this->thankyouUrl)
		             ->setCancelUrl($this->cancelUrl);
		// Create the full payment object
		$payment = new Payment();
		$payment->setIntent('sale')
		        ->setPayer($payer)
		        ->setRedirectUrls($redirectUrls)
		        ->setTransactions(array($transaction));
		// Create payment with valid API context
		try {
			$payment->create($this->oApiContext);
			// Get PayPal redirect URL and redirect user
			$approvalUrl = $payment->getApprovalLink();
			$this->parseTokenFromApprovalUrl($approvalUrl);
			// Insert wiloke_submission_transaction and wiloke_submission_paypal_nonrecurring_payment before redirecting to PayPal
			$sessionID = PaymentModel::insert($this, $this->oReceipt);

			if ( empty($sessionID) ){
				return array(
					'status' => 'error',
					'msg'    => esc_html__('We could not create this session', 'wiloke')
				);
			}else{
				$this->storeTokenAndPlanId();
				return array(
					'status'    => 'success',
					'msg'       => esc_html__('Got Approval url', 'wiloke-submission'),
					'next'      => $approvalUrl,
					'redirectTo'=> $approvalUrl
				);
			}
		} catch (PayPalConnectionException $ex) {
			return array(
				'code'   => $ex->getCode(),
				'status' => 'error',
				'msg'    => $ex->getMessage()
			);
		} catch (\Exception $ex) {
			return array(
				'status' => 'error',
				'msg'    => $ex->getMessage()
			);
		}
	}
}