<?php

class WilokeMenu_Walker_Nav_Menu extends Walker_Nav_Menu
{
    /**
     * Starts the list before the elements are added.
     *
     * @since 3.0.0
     *
     * @see Walker::start_lvl()
     *
     * @param string $output Passed by reference. Used to append additional content.
     * @param int    $depth  Depth of menu item. Used for padding.
     * @param array  $args   An array of wp_nav_menu() arguments.
     */
    
    public static $css = '';

    public function start_lvl(&$output, $depth = 0, $args = array())
    {

        $attr = '';

        if (isset($args->options['animate']) && !empty($args->options['animate'])) {
            $attr .= 'data-animated="' . esc_attr($args->options['animate']) . '" ';
        }

        $indent = str_repeat("\t", $depth);

        if (isset($args->setting)) {

            $setting = $args->setting;

            if (isset($setting['width'])) {
                $attr .= 'data-width="' . esc_attr($setting['width']) . '" ';
            }

            if (isset($setting['align'])) {
                $attr .= 'data-align="' . esc_attr($setting['align']) . '" ';
            }

            if ( isset($setting['background_color']) && !empty($setting['background_color']) ) {
                $attr .= 'style="background-color: '. esc_attr($setting['background_color']) .'"';
            }

            $attr = trim($attr);
        }

        $output .= "\n$indent<ul class=\"wiloke-menu-sub wiloke-menu-sub-dropdown\" $attr>\n";
    }

    /**
     * Starts the element output.
     *
     * @since 3.0.0
     * @since 4.4.0 The {@see 'nav_menu_item_args'} filter was added.
     *
     * @see Walker::start_el()
     *
     * @param string $output Passed by reference. Used to append additional content.
     * @param object $item   Menu item data object.
     * @param int    $depth  Depth of menu item. Used for padding.
     * @param array  $args   An array of wp_nav_menu() arguments.
     * @param int    $id     Current item ID.
     */
    public function start_el(&$output, $item, $depth = 0, $args = array(), $id = 0)
    {
        // Setting Megamenu
        $setting = get_post_meta($item->ID, 'wiloke-nav-item-data', true);

        $indent = ($depth) ? str_repeat("\t", $depth) : '';

        $classes   = empty($item->classes) ? array() : (array) $item->classes;
        $classes[] = 'menu-item-' . $item->ID;

        if( $setting && isset($setting['megamenu']) ) {
            $classes[] = 'menu-item-has-children';
        }

        /**
         * Filters the arguments for a single nav menu item.
         *
         * @since 4.4.0
         *
         * @param array  $args  An array of arguments.
         * @param object $item  Menu item data object.
         * @param int    $depth Depth of menu item. Used for padding.
         */
        $args = apply_filters('nav_menu_item_args', $args, $item, $depth);

        /**
         * Filters the CSS class(es) applied to a menu item's list item element.
         *
         * @since 3.0.0
         * @since 4.1.0 The `$depth` parameter was added.
         *
         * @param array  $classes The CSS classes that are applied to the menu item's `<li>` element.
         * @param object $item    The current menu item.
         * @param array  $args    An array of wp_nav_menu() arguments.
         * @param int    $depth   Depth of menu item. Used for padding.
         */
        $class_names = join(' ', apply_filters('nav_menu_css_class', array_filter($classes), $item, $args, $depth));
        $class_names = $class_names ? ' class="' . esc_attr($class_names) . ' wiloke-menu-item wiloke-menu-item-level-' . esc_attr($depth) . '"' : '';

        /**
         * Filters the ID applied to a menu item's list item element.
         *
         * @since 3.0.1
         * @since 4.1.0 The `$depth` parameter was added.
         *
         * @param string $menu_id The ID that is applied to the menu item's `<li>` element.
         * @param object $item    The current menu item.
         * @param array  $args    An array of wp_nav_menu() arguments.
         * @param int    $depth   Depth of menu item. Used for padding.
         */
        $id = apply_filters('nav_menu_item_id', 'menu-item-' . $item->ID, $item, $args, $depth);
        $id = $id ? ' id="' . esc_attr($id) . '"' : '';

        $output .= $indent . '<li' . $id . $class_names . '>';

        $atts           = array();
        $atts['title']  = !empty($item->attr_title) ? $item->attr_title : '';
        $atts['target'] = !empty($item->target) ? $item->target : '';
        $atts['rel']    = !empty($item->xfn) ? $item->xfn : '';
        $atts['href']   = !empty($item->url) ? $item->url : '';

        /**
         * Filters the HTML attributes applied to a menu item's anchor element.
         *
         * @since 3.6.0
         * @since 4.1.0 The `$depth` parameter was added.
         *
         * @param array $atts {
         *     The HTML attributes applied to the menu item's `<a>` element, empty strings are ignored.
         *
         *     @type string $title  Title attribute.
         *     @type string $target Target attribute.
         *     @type string $rel    The rel attribute.
         *     @type string $href   The href attribute.
         * }
         * @param object $item  The current menu item.
         * @param array  $args  An array of wp_nav_menu() arguments.
         * @param int    $depth Depth of menu item. Used for padding.
         */
        $atts = apply_filters('nav_menu_link_attributes', $atts, $item, $args, $depth);

        $attributes = '';
        foreach ($atts as $attr => $value) {
            if (!empty($value)) {
                $value = ('href' === $attr) ? esc_url($value) : esc_attr($value);
                $attributes .= ' ' . $attr . '="' . $value . '"';
            }
        }

        /** This filter is documented in wp-includes/post-template.php */
        $title = apply_filters('the_title', $item->title, $item->ID);

        /**
         * Filters a menu item's title.
         *
         * @since 4.4.0
         *
         * @param string $title The menu item's title.
         * @param object $item  The current menu item.
         * @param array  $args  An array of wp_nav_menu() arguments.
         * @param int    $depth Depth of menu item. Used for padding.
         */
        $title = apply_filters('nav_menu_item_title', $title, $item, $args, $depth);

        /*
         *   Menu Item Setting
         */

        if ($setting && !isset($setting['megamenu'])) {
            $args->setting = $setting;
        } else {
            unset($args->setting);
        }

        $item_output = $args->before;
        $item_output .= '<a' . $attributes . '>';

        if (isset($setting['icon']) && !empty($setting['icon'])) {
            $item_output .= '<i class="wiloke-menu-item-icon ' . esc_attr($setting['icon']) . '"></i>';
        }

        $item_output .= $args->link_before . $title . $args->link_after;
        $item_output .= '</a>';
        $item_output .= $args->after;

        /**
         * Filters a menu item's starting output.
         *
         * The menu item's starting output only includes `$args->before`, the opening `<a>`,
         * the menu item's title, the closing `</a>`, and `$args->after`. Currently, there is
         * no filter for modifying the opening and closing `<li>` for a menu item.
         *
         * @since 3.0.0
         *
         * @param string $item_output The menu item's starting HTML output.
         * @param object $item        Menu item data object.
         * @param int    $depth       Depth of menu item. Used for padding.
         * @param array  $args        An array of wp_nav_menu() arguments.
         */

        // Render content with visual composer
        if ($setting && isset($setting['megamenu'])) {

            $attr = '';
            $sub_class = 'wiloke-menu-sub-' . esc_attr($item->ID);

            if (isset($setting['width'])) {
                $attr .= 'data-width="' . esc_attr($setting['width']) . '" ';
            }

            if (isset($setting['align'])) {
                $attr .= 'data-align="' . esc_attr($setting['align']) . '" ';
            }

            if (isset($args->options['animate']) && !empty($args->options['animate'])) {
                $attr .= 'data-animated="' . esc_attr($args->options['animate']) . '" ';
            }

            if ( isset($setting['background_color']) && !empty($setting['background_color']) ) {
                $attr .= 'style="background-color: '. esc_attr($setting['background_color']) .'"';
            }

            $attr = trim($attr);

            $item_output .= "\n$indent<div class=\"wiloke-menu-sub wiloke-menu-sub-mega $sub_class\" $attr>\n";
            
            $item_output .= wpb_js_remove_wpautop($setting['content']);
            
            $item_output .= "\n$indent</div>\n";

            self::$css .= visual_composer()->parseShortcodesCustomCss($setting['content']);
        }

        $output .= apply_filters('walker_nav_menu_start_el', $item_output, $item, $depth, $args);

        /**
         * Render content visual
         *
         */
    }

} // Walker_Nav_Menu
